/*/////////////////////////////////////////////////////////
USAGE:

  This application is used to provide a STDOUT for Windows GUI
    applications for use as CGI applications.

PARAMS:
    <FILENAME>.EXE

    Can be passed either on the command line or as the first
    (and currently only) parameter in the QUERY_NAME
    environment variable.

    The EXE can be passed up to two parameters (see below.)

      Param 1:  Name of the Temporary File to create and use for passing
                information back to this application.

      Param 2 (Optional): Generally used for Drill-down capabilities

PARAMETERS:

    A single paramater can be passed by sending the EXE on the command line
    (not using 'GET'), using a '~' or '&' as a delimiter between the
    EXE name and the parameter.

INTERNALS:
    This application will generate a unique file name, pass it as a
    command-line argument to an application, wait for the called application
    to terminate, and will then output the contents of the file to
    STDOUT.

TODO:

  Add 'trace' option - output goes to file....
  Test
  Multi-User Tests
  Internationalize
  Handle both 'GET' and 'POST'

  How to handle multiple errors without multiple "content-type" lines 
  

DONE

  12/18/97
    Changed to use GetTempFileName to determine temporary filename
    Added automatic display of CGI header information
      - Added check to make certain CGI Header Information is displayed only once


  11/23/97
    Display only filename (in place of command line) in errors regarding
      launching the application
      
  11/22/97
    Error Handler to use file for HTML to display
    Parse paramater list (smarter parsing)
    Validate requested EXE and display reasonable error
    Single Exit Point from main
    Add support for second parameter
    Add Time Out (if called application no come back....)
*/

#include <dos.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <win32\windows.h>
#pragma hdrstop

// Global Error Handler
int (*errFunc)(char *);

// CONSTANTS:
//
// Assumed length of largest command line or env variable
int cgiCmdLength = 32768; // Assumed Largest length of
                          //   a single line of CGI
DWORD timeOut = 120000;   // Default Timeout Value - how long to wait for
                          //   the VdB Application to finish
int headerDisplayed = 0;

// Used for a single exit point from the 'main' function
#define END_CLEANUP goto cleanup;

// Function Prototypes
int displayFile( char *fileName, FILE * fileStream );
int displayHeader(void);
int parseCommandLine( char *cmdLine, char * parseString, char *tmpFile, char *exeFileName );
int stdErrHandler( char * szErrMessage );
int cgiErrHandler( char * szErrMessage );

#pragma argsused
int main (int argc, char *argv[])
{
  PROCESS_INFORMATION pi = {0}; // Process Information - used to get
                                //   thread handle to determine when VdB
                                //   application shuts down
  STARTUPINFO si = {0};         // No Longer Used
  DWORD status;                 // On Error from CreateProcess

  char * lpCmdLine     = NULL;  // Holds command line to send to
                                //   VdB Application
  char * lpFileName    = NULL;  // File name from command line
  char * qryStringTemp = NULL;  // Temp variable for parsing QUERY_STRING
                                // Environment Variable
  char * tmpFileName   = NULL;  // File name used to get information from EXE

  char * szError       = NULL;  // Error buffer
  char * szSysMsg      = NULL;  // Used to get System Error message
  char exeFileName[256];        // Store the filename of the EXE to call

  int  retry;                   // Retry value
  FILE *tmpFile;

  // Assign error handler based on CGI Environment Variable
  if (getenv("HTTP_USER_AGENT") == NULL) {
    errFunc = stdErrHandler;
  } else {
    errFunc = cgiErrHandler;
  }

  // Initialize FileName
  strcpy(exeFileName, "(None)");

  // Set size of Startup Info Structure
  si.cb = sizeof (si);

  // Allocate memory for the command line
  if ((lpCmdLine = (char *) malloc(cgiCmdLength * sizeof(char))) == NULL) {
    errFunc("Could not allocate memory\n");
    END_CLEANUP
  }
  // Initialize to blank line
  strcpy(lpCmdLine, "");

  // Allocate memory for the temporary File Name
//  if ((tmpFileName = (char *) malloc(512 * sizeof(char))) == NULL) {

  if ((tmpFileName = (char *) malloc(MAX_PATH * sizeof(char))) == NULL) {
    errFunc("Could not allocate memory\n");
    END_CLEANUP
  }

  // Generate a temporary File Name for passing data from the
  //  VdB Application back to this application.

/*
  retry = 0;
  do {
    lpFileName = tmpnam(NULL);

    if ((tmpFile = fopen(lpFileName, "r+t")) == NULL) {
      // Create file if it doesn't exist
      if ((tmpFile = fopen(lpFileName, "w+t")) != NULL) {
        retry = 10; // Exit the loop
      }
    } else {
      // Need to get a different name
      retry++;
      sleep(0.2);
    }
    // Close the file Handle
    fclose(tmpFile);
  } while ((tmpFile == NULL) && (retry < 10));

  if (lpFileName != NULL) {
    strcpy(tmpFileName, lpFileName);
  }
*/
  if (GetTempFileName(".", "CGI", 0, tmpFileName) == 0) {
    if ((szSysMsg = (char *)malloc(512 * sizeof(char))) != NULL) {
      if (!FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError(), 0, szSysMsg, 512, NULL)) {
        // If no message from FormatMEssage....
        if ((szError = (char *)malloc(4096 * sizeof(char))) != NULL) {
          sprintf(szError, "Could not create temporary file. File Name: %s\n", tmpFileName);
          errFunc(szError);
          free(szError);
        }
      } else {
        if ((szError = (char *)malloc(4096 * sizeof(char))) != NULL) {
          sprintf(szError, "%s   File Name: %s\n", szSysMsg, exeFileName);
          errFunc(szError);
          free(szError);
        }
      }
      free(szSysMsg);
    } else {
      errFunc("Could not create temporary filename\n");
    }
    END_CLEANUP
  }

  //
  // Set up the Command Line (lpCmdLine) variable.
  //
  // If EXE name is passed on the command line...
  if (argc > 1) {
    parseCommandLine( lpCmdLine, argv[1], tmpFileName, exeFileName );
  } else {
    // If not on command line, but through CGI in the environment...
    qryStringTemp = getenv("QUERY_STRING");
    if (qryStringTemp != NULL) {
      parseCommandLine( lpCmdLine, qryStringTemp, tmpFileName, exeFileName );
    } else {
      errFunc("An EXE name is required: FILENAME.EXE");
      END_CLEANUP
    }
  }

#ifdef DEBUG
  if ((szError = (char *)malloc(4096 * sizeof(char))) != NULL) {
    sprintf(szError, "\n\nIn Main:\n  Command Line: %s\nFile Name: %s\n\n", lpCmdLine, tmpFileName);
    errFunc(szError);
    free(szError);
  }
#endif

  // Start the VdB application
  if (CreateProcess (NULL, lpCmdLine, NULL, NULL, TRUE, HIGH_PRIORITY_CLASS, NULL, NULL, &si, &pi) == 0)
  {
    if ((szSysMsg = (char *)malloc(512 * sizeof(char))) != NULL) {
      if (!FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError(), 0, szSysMsg, 512, NULL)) {
        // If no message from FormatMEssage....
        if ((szError = (char *)malloc(4096 * sizeof(char))) != NULL) {
          sprintf(szError, "Could not launch application. File Name: %s\n", exeFileName);
          errFunc(szError);
          free(szError);
        }
      } else {
        if ((szError = (char *)malloc(4096 * sizeof(char))) != NULL) {
          sprintf(szError, "%s   File Name: %s\n", szSysMsg, exeFileName);
          errFunc(szError);
          free(szError);
        }
      }
      free(szSysMsg);
    } else {
      errFunc("Could not launch application\n");
    }
    END_CLEANUP
  }

  // Wait for called VdB Application to terminate
  status = WaitForSingleObject (pi.hProcess, timeOut);
  switch (status) {
    case WAIT_FAILED:
      if ((szSysMsg = (char *)malloc(512 * sizeof(char))) != NULL) {
        if (!FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError(), 0, szSysMsg, 512, NULL)) {
          // If no message from FormatMEssage....
          if ((szError = (char *)malloc(4096 * sizeof(char))) != NULL) {
            sprintf(szError, "Could not Retreive Information. File Name: %s\n", exeFileName);
            errFunc(szError);
            free(szError);
          }
        } else {
          if ((szError = (char *)malloc(4096 * sizeof(char))) != NULL) {
            sprintf(szError, "Could not retrieve information.%s   File Name: %s\n", szSysMsg, exeFileName);
            errFunc(szError);
            free(szError);
          }
        }
        free(szSysMsg);
      } else {
        errFunc("Could not retrieve information\n");
      }
      END_CLEANUP
      break;
    case WAIT_TIMEOUT:
    case WAIT_ABANDONED:
      errFunc("System Busy - Timeout Value Exceeded\n");
      END_CLEANUP
      break;
  }

#ifdef DEBUG
  if ((szError = (char *)malloc(4096 * sizeof(char))) != NULL) {
    sprintf(szError, "\n\nIn Main:\n  Command Line: %s\nAbout to open File Name: %s\n\n", lpCmdLine, tmpFileName);
    errFunc(szError);
    free(szError);
  }
#endif

  displayFile( tmpFileName, stdout );

  if (remove(tmpFileName) == -1) {
    if ((szError = (char *)malloc(4096 * sizeof(char))) != NULL) {
      sprintf(szError, "Could not delete temp file: %s", tmpFileName);
      errFunc(szError);
      free(szError);
    }
  }

cleanup:  
  if (lpCmdLine != NULL)
    free(lpCmdLine);
  if (tmpFileName != NULL)
    free(tmpFileName);
  return 0;
}

//
// displayFile( char *fileName, FILE *fileStream )
//
//   Routine to output results of a file to STDOUT
//
int displayFile( char *fileName, FILE * fileStream ) {

  FILE * cgi_commands = NULL;   // FILE handle for input Temp File
  char * cgiCmdLine;            // The CGI command line from the Temp file
  char * displayString;         // Used for displaying debug / error info
  int  openCount = 0;           // Retry variable for opening file
  char testString[10];

#ifdef DEBUG
  if ((displayString = (char *)malloc(4096 * sizeof(char))) != NULL) {
    sprintf(displayString, "Function: displayFile. FileName: %s\n", fileName);
    errFunc(displayString);
    free(displayString);
  }
#endif

  // Open the temporary file for reading in the CGI commands from the VdB
  //   application - complicated because it needs to check if the data
  //   has been written - 'flush' in VdB does not imidiately 'flush'....
  do {
    cgi_commands = fopen(fileName, "rt");
    openCount++;
    if (cgi_commands == NULL) {
      sleep(0.2 * openCount); // Increase wait time - VdB Issues....
    } else {
      if ((fgets(testString, 9, cgi_commands)) == NULL) {
        // Data not yet written - wait to try again....
        fclose(cgi_commands);
        cgi_commands = NULL;
        sleep(0.2 * openCount); // Increase wait time - VdB Issues....
      } else {
        fseek(cgi_commands, 0, SEEK_SET);
      }
    }
  } while ((cgi_commands == NULL) && (openCount < 10));

  if (cgi_commands == NULL) {
     if ((displayString = (char *)malloc(4096 * sizeof(char))) != NULL) {
       sprintf(displayString, "Did not receive response data from application."
                              " File: %s ReTry: %d\n", fileName, openCount);
       errFunc(displayString);
       free(displayString);
       return -1;
     }
  } else {
    if ((cgiCmdLine = (char *) malloc(cgiCmdLength * sizeof(char))) == NULL) {
      return errFunc("Could not allocate memory for reading response data\n");
    }

    displayHeader();
    // Display contents of file
    while (!feof(cgi_commands)) {
      if ((fgets(cgiCmdLine, cgiCmdLength - 1, cgi_commands)) != NULL) {
        fprintf(fileStream, cgiCmdLine);
      }
    }

    free(cgiCmdLine);
    fclose(cgi_commands);
  }

#ifdef DEBUG
  if ((displayString = (char *)malloc(4096 * sizeof(char))) != NULL) {
    sprintf(displayString, "Function: displayFile. FileName after Parsing: %s RetryCount: %d\n", fileName, openCount);
    errFunc(displayString);
    free(displayString);
  }
#endif

  return 0;
}

//
// parseCommandLine( char *cmdLine, char * parseString )
//
//   Parses paramaters that are passed in.
//
int parseCommandLine( char *cmdLine, char * parseString, char *tmpFile, char *exeFileName ) {

  char  delimiters[] = "~&";    // Deliters for STRTOK

  char * qryName;               // Value returned from STRTOK
  char * qryDrillDown = NULL;   // Possible Second Paramater    
  char * temp;                  // Temporary Variable used for STRTOK
  char * exeName;               // Variable to store EXE name
  char * test;
  
#ifdef DEBUG
  if ((test = (char *)malloc(4096 * sizeof(char))) != NULL) {
    sprintf(test, "Function: parseCommandLine. At Start. File Name: %s\n\n", tmpFile); 
    errFunc(test); 
    free(test);
  }
#endif

  // Determine EXE FileName and possible second parameter
  if (parseString != NULL) {
    // Test if the EXEFILENAME is specified in the string
    if ((exeName = strstr(parseString, "EXEFILENAME=")) == NULL) {
      qryName = strtok(parseString, delimiters);
      qryDrillDown = strtok(NULL, delimiters); // Check for second paramenter
    } else { // If result of 'get'
      exeName = strchr(parseString, '='); 
      if (exeName != NULL) {
        // Get the EXENAME paramater
        temp = strtok(&exeName[1], "&");
        // Check if the EXENAME includes 'filename' and 'Paramater'
        if (temp != NULL) {
          qryName = strtok(temp, "~");
          qryDrillDown = strtok(NULL, "~");
        }
      }
    }
  }
  // Create command line arguments
  if (qryName != NULL) {
    temp = stpcpy(cmdLine, qryName); // Copy the FileName to the command line
    strcpy(exeFileName, qryName); // EXE FileName to return to parent
    temp = stpcpy(temp, " ");

    if (tmpFile != NULL) {
      temp = stpcpy(temp, tmpFile); // Copy the Temporary filename to the
                                    //   command line
    }

    if (qryDrillDown != NULL) {   // Copy second parameter
      temp = stpcpy(temp, " ");
      stpcpy(temp, qryDrillDown);
    }
  }

#ifdef DEBUG
  if ((test = (char *)malloc(4096 * sizeof(char))) != NULL) {
    sprintf(test, "Function: parseCommandLine. At End. Command Line: %s\nFile Name: %s\n\n", cmdLine, tmpFile); 
    errFunc(test); 
    free(test);
  }
#endif

  return 0;
}      

//
// stdErrHandler(char * szErrMessage)
//
//   Handles errors to STDERR - for testing purposes
//     
int stdErrHandler(char * szErrMessage) {
  fprintf(stdout, "VDB_CGI Error:\n  ");
  fprintf(stdout, szErrMessage);
  return -1;
}

//
// cgiErrHandler(char * szErrMessage)
//
//   Handles errors to the Browser
//
//     Add handler for file - if 'error.html' in same directory, 
//     read and display.
//
int cgiErrHandler(char *szErrMessage) {

  FILE *cgi_commands;          // FILE handle to read from Temp file               
  char *cgiCmdLine;            // The CGI command line from the Temp file

  // Open the temporary file for reading in the CGI commands from the VdB 
  //   application
  if ((cgi_commands = fopen("error.htm", "rt")) == NULL) {
     // If file does not exist, display standard error
    displayHeader();
    printf("<HTML>\n");
    printf("<HEAD>\n");
    printf("<TITLE>Simple Html Page</TITLE>\n");
    printf("</HEAD>\n");
    printf("<BODY>\n");
    printf("<CENTER><H1><FONT COLOR=\"RED\">VDB_CGI Message</FONT></H1></CENTER>\n");
    printf("<CENTER><H3>%s</H3></CENTER>\n", szErrMessage);
    printf("<p></p><p></p>\n");
    printf("</BODY>\n");
    printf("</HTML>\n");
  } else {
    if ((cgiCmdLine = (char *) malloc(cgiCmdLength * sizeof(char))) == NULL) {
      return errFunc("Could not allocate memory for reading response data\n");
    }

    displayHeader();

    while (!feof(cgi_commands)) {
       fgets(cgiCmdLine, cgiCmdLength - 1, cgi_commands);
       // Check if string contains formating character
       if (strstr(cgiCmdLine, "%s")) {
         printf(cgiCmdLine, szErrMessage);
       } else {
         printf(cgiCmdLine);
       }
    }

    free(cgiCmdLine);
    fclose(cgi_commands);
  }

  return -1;
}

//
// displayHeader( )
//
//   Routine to output standard CGI Header
//

int displayHeader(void) {
  // Write header to STDOUT
  if (!headerDisplayed) {
    printf("Content-type: text/html\n\n");
    // Set flag that header has been displayed
    headerDisplayed = 1;
  }
  return 0;
}




