/* Wizout.cc  	Output classes for Borland's Visual dBASE 7 Web Wizard
					(c) 1997 Borland International, all rights reserved

   Main Classes: 
               StreamHTML streams HTML starting pages and static "publish"
               reports.

               StreamCGI streams .prg source code, compiles to .pro and
               builds an executable for server-side processing.

   Purpose		These classes perform operations for the three application types
               supported by the Web Wizard

               Data-Entry Application
               Streams starting HTML page with form and fields 
               Streams .prg for CGI back-end processing (updating rowset)
               Compiles .prg and Builds executable to run server-side

               Query and Response Application
               Streams starting HTML page with initial search criteria control
               Modifies .REP file specified by user
               Generates .prg for CGI back-end processing (run report)
               Modifies second .REP file to respond to Href from first report

               Publish Application
               Modifies report properties to generate HTML Output to 
               specified web site folder
               Runs report Output to web site folder

    Note: 		How streamed code is called:
               All CGI interactive apps use a "shell" .exe on the server
               named VdB_Cgi. No VdB executable is called directly. The
               name of the VdB Exe can be sent either in an HTML Hidden
               (name ="EXEFILENAME", value="X.exe", where X is the name
               of the generated Visual dBASE app. This code is streamed
               automatically by the Wizards in the HTML pages they
               generate 

               However, there are two cases where the .exe must be called
               directly, without an HTML "Form" that supports a "Hidden".
               In that case, VdB_CGI gets called with at least one param - 
               the name of the VdB executable:

               http://www.mydomain/cgi-bin/vdb_cgi.exe?Myprogram.exe

    Author		A.A.Katz, KSoft, Inc. under contract to Borland

    Versions:	Version 1.0 10-14-1997	

*/


 /////////////////////////////////////////////////////////////////////////////
 //////////Class 		Wizout///////////////////////////////////////////////////
 //////////Purpose:  Instantiates object, receives parent form      //////////
 /////////////////   object reference
 /////////////////////////////////////////////////////////////////////////////

 class WizOut(oForm)
 this.cCurrPath = Set('Directory') // store path
 this.oForm = oForm
 this.bCopyAll = false  
 this.cReportPath = ''
 this.cReportName = ''
 this.cCgiFolder = ''
 this.cHTMLFolder = ''
 this.cClass = ''
 this.cOutputFileName = ''

 ///////////////////////Method: StreamHTML///////////////////////////////////
 ///////////////////////Purpose: Generates starting HTML page ///////////////
 ///////////////////////Returns: True, if successful/////////////////////////

 /* Note: StreamHTML has four sections in the following order:
          Live Publish Reports
          Static Publish Reports // do nothing
          Data-Entry
          Query
 */

 function StreamHTML()


    ////// Note: no HTML is streamed for live Publish Reports
    ////// It is streamed for published reports (the report itself)
    ////// plus starting pages for Query and Data-Entry 
    
    ////// 1.Publish Reports

    If this.oForm.cWizardType = 'Publish'
       this.oForm.Progress1.value = 10

       ////////////////////////// Get paths from fields on forms

       ////// Extract Report path
       this.cReportPath = trim(this.oForm.PublishReportField.value)

       ////// Extract Report Name
       if at('\',this.cReportPath) > 0
          this.cReportName = substr(this.cReportPath,;
               rat('\',this.cReportPath)+1)
       else
          this.cReportName = this.cReportPath

       endif


       if at('.',this.cReportName) > 0
          this.cReportName = substr(this.cReportName,1,;
               at('.',this.cReportName)-1)

       endif

       
       ///////////////////// Set up target paths for "publish" reports

       /* Note: the same field (form.cGiPathField) is used for 
                both HTMLpath and CGIpath in the "Publish" wizard. 
                Live reports use the field as CGIpath (no HTML required)
                Static reports use the field as HTMLpath (no CGI required)
       */

       ////// Set HTML folder
       this.cHtmlFolder = trim(this.oForm.PublishCGIField.value)
       if right(this.cHtmlFolder,1) # '\'
          this.cHtmlFolder += '\'
       endif

       ////// Set CGI path
       this.cCgiFolder = this.cHtmlFolder  // Path used for both, see note.

       ////// Set output path
       this.cOutPutPath = this.cHtmlFolder+this.cReportName+'.htm'      
       this.cOutputFileName = this.cReportName


       ////////////////////// Open report file and get classname
     

       if this.oForm.LiveReportRadio.value

          this.oForm.Progress1.value = 90  // Finish and reset progress bar    
          this.oForm.Progress1.value = 0

          ////// Live reports are done, no HTML to stream
          return true 

       endif

      
       ////////////////////// Static REports

       this.oForm.Progress1.value = 40  // update progress bar


       ////// Try to open report
       fRep = new File() // create low-level file object

       try

          fRep.Open(this.cReportPath,"R")
         
       catch(Exception e)

          // if fail, sorry...
          msgbox(e.Message,'Report Failed to Open',16)
          return false

       endtry

      
       ////// Extract report class name
       this.cClass = ''

       do
          ////// Get next line in .rep file         
          cStr = Upper(fRep.Gets())

          ////// if it's the class declaration line         
          if at('CLASS',cStr) > 0 and at('OF',cStr) > 0 and;
             at('REPORT',cStr) > 0
             
             ////// parse out the class name
             this.cClass = trim( ltrim(  substr(cStr,at('CLASS',cStr)+5) ) )
             this.cClass = trim( ltrim(  substr(this.cClass,1, at(' OF ',this.cClass)-1) ) )

             exit

          endif

       until fRep.eof() // until the end of the report file

       ////// Clean up file object
       fRep.Close()
       fRep = ''

       this.oForm.Progress1.value = 70   // update progress bar

       ////// If no class declaration found
       if len(trim(this.cClass)) = 0

          msgbox('Unable to find Report Class in '+this.cReportName,;
                 'Cannot find class',16)
          return false

       endif


       ////// try to instantiate the report class
       try

         cClass = this.cClass
         set proc to (this.cReportPath) addi   
         this.oRep = new &cClass.()

       catch (Exception e)

          ////// if fail, go back
          msgbox(e.Message+chr(13)+;
                'Report may have dependencies or'+chr(13)+;
                'report may require explicit paths in queries or' +chr(13)+;
                'report may require anonymous login for remote server.',;
                'Report Instantiation Failed',16)

          return false

       EndTry

       this.oForm.Progress1.value = 80  // update progress bar

       ////// Set properties of report class to stream as HTML
       this.oRep.Output = 4 // HTML
       this.oRep.OutputFileName = this.cOutPutPath

       ////// Try to stream the report
       try
          this.oRep.Render()

       catch (Exception e)

          msgbox('Error rendering report'+chr(13)+;
                  e.message,'Report Error',16)
          return false

       endtry 

       this.oForm.Progress1.value = 100 // update progress bar

       ////// Clean up report
       Close Proc (this.cReportPath)
       this.oRep = ''

       this.oForm.Progress1.value = 0
       return true   // static report streamed successfully
                     // All "Publish" reports are done                     
    endif

   

    //////////////// 2. and 3.:Wizards that require HTML pages
    ////////////////          (Query and Data-Entry)

    ////// reset "yes to all" for "duplicate" dialog
    this.bCopyAll = false  
    ////// Counter for loops
    local n     
    ////// Image Path variables
    this.BackGroundImage = ''
    this.LogoImage = ''
    this.MailImage = ''
    this.oForm.cHomeImage = ''

  
    ////// Output File Name from user input
    this.cOutputFileName = ltrim(trim(this.oForm.FileNameField.value))
  

    if at('.',this.cOutputFileName) > 0
       this.cOutputfileName = substr(this.cOutputFileName,1,;
                              at('.',this.cOutputFileName)-1)
    endif

    if at('/',this.cOutputFileName) > 0
       this.cOutputFileName = substr(this.cOutputfileName,;
                           rat('\',this.cOutputFileName)+1)
    endif


    ////// Clean up URL and set CGI EXE Name with full web path
    this.cCGIUrl = ltrim(trim(this.oForm.UrlField.value))
    if substr(this.cCgiUrl,len(this.cCgiUrl),1) # '/'
       this.cCgiUrl += '/'
    endif
    if substr(upper(this.cCgiUrl),1,7) # 'HTTP://'
       this.cCgiUrl = 'http://'+this.cCgiUrl
    endif


    ////// Clean up CGI folder
    this.cCgiFolder = this.oForm.CGIPathField.value
    if empty(this.cCgiFolder)
       return false
    endif
    this.cCgiFolder = Class::CleanUpFolder(this.cCgiFolder)

    ////// Paste up CgiFileName
    this.cCgiFileName = this.cCgiFolder+this.cOutputFileName+'.exe'
    //CGI path is formatted properly and tested


    /////// Clean up HTML path
    this.cHtmlFolder = this.oForm.HTMLPathField.value
    if empty(this.cHtmlFolder)
       msgbox('Web Site HTML path required to finish application.')
       return false
    endif
    this.cHtmlFolder = Class::CleanUpFolder(this.cHtmlFolder)

    ////// Set HTML file name
    this.cHtmlFileName = this.cHtmlFolder+this.cOutputFileName+'.htm'

    ////// test for duplicates 
    if file(this.cHTMLFileName)
       if msgbox('Filename: '+this.cHTMLFileName+' already exists! '+;
          'Overwrite?','Duplicate Filename',36) # 6
          return false
       endif
       dele file (this.cHTMLFileName)
    endif

    ////// Set Report Path
    If this.oForm.cWizardType = 'Query'                        
       this.cReportPath = trim(this.oForm.FirstReportField.value)
    else
       this.cReportPath = trim(this.oForm.FileNameField.value)
    endif

    ////// extract report name
    if at('\',this.cReportPath) > 0
       this.cReportName = substr(this.cReportPath,;
            rat('\',this.cReportPath)+1)
    else
       this.cReportName = this.cReportPath
    endif  

    ////// remove extensions
    if at('.',this.cReportName) > 0
       this.cReportName = substr(this.cReportName,1,;
            at('.',this.cReportName)-1)
    endif   
    
    
    //////////// Copy in graphic images for background, home and mail
    //////////// Note: bad files or paths are not treated as critical
    //////////// errors since the Browser just puts a "broken image"
    //////////// logo on the HTML page. Warnings, however, are given

    //////////// each image is removed from datasources so that it can
    //////////// be copied, and then restored when done.

    this.oform.progress1.value = 10 // update progress bar

    ////// Copy graphic files to target folders

    cd (this.oForm.cLoadPath)  // go back to home directory

    /////// background images
    ////// store datasources
    cBkg = this.oForm.Background
    cDsc = this.oForm.BackgroundDisplay.DataSource

    this.oForm.Background = 'FILENAME Blank.gif'
    this.oForm.BackgroundDisplay.DataSource = 'FILENAME Blank.gif'
    
    ////// copy files
    this.BackgroundImage = class::CopyGraphicFile(this.oForm.BackgroundImageField.value)
    // copy to CGI folder too...
    class::CopyGraphicFile(this.oForm.BackgroundImageField.value,this.cCgiFolder)

    ////// restore datasources
    this.oForm.Background = cBkg
    this.oForm.BackgroundDisplay.DataSource = cDsc
   

    ////// copy home image
    cDsc = this.oForm.HomeImage.DataSource
    this.oForm.HomeImage.DataSource =  'FILENAME Blank.gif'
               
    this.oForm.cHomeImage = class::CopyGraphicFile(this.oForm.HomeImageField.value)
    this.oForm.HomeImage.DataSource = cDsc
        

    ////// mail image
    cDsc = this.oForm.MailImage.DataSource
    this.oForm.MailImage.DataSource = 'FILENAME Blank.gif'
          
    this.MailImage = class::CopyGraphicFile(this.oForm.MailImageField.value)
    this.oForm.MailImage.DataSource = cDsc
          

    ////// logo image
    cDsc = this.oForm.LogoImage.DataSource
    this.oForm.LogoImage.DataSource = 'FILENAME Blank.gif'       
    this.LogoImage			= class::CopyGraphicFile(this.oForm.LogoImageField.value)
           this.oForm.LogoImage.DataSource = cDsc
         

    this.oform.progress1.value = 40  // update progress bar


    ////// Stream HTML astarting page

    if file(this.cHtmlFileName)
       dele file (this.cHtmlFileName)
    endif

    fHtml = new File()

    Try

       fHtml.Create(this.cHtmlFileName,"RA")

    catch (Exception e)

       msgbox('Cannot create: '+this.cHtmlFileName+ chr(13)+e.message )
       return false
    endtry


    ////// Stream out Header

    fHtml.Puts('<HTML>')
    fHtml.Puts('')
    fHtml.Puts('<head>')
    fHtml.Puts('<meta http-equiv="Content-Type" content="text/html">')
    fHtml.Puts('<meta name="Generator" content="Visual dBASE 7 Web Wizard">')
    if not empty(this.oForm.FirstPageTitle.Editor1.value)
       fHtml.Puts('<title>'+this.oForm.FirstPageTitle.Editor1.value+'</title>')
    endif
    fHtml.Puts('</head>')
    fHtml.Puts('')

    this.oform.progress1.value = 50

    /////// stream out body tag
    fHtml.Puts('   '+class::BuildBodyString())
    fHtml.Puts('')
    fHtml.Puts('   <blockquote>')

    class::StreamTitleAndLogo()

    ////// put in text above data-entry area
    fHtml.Puts('')
    cHtml = class::BuildHTMLString(this.oForm.TextAboveDataEntry.Editor1.value)
    if not empty(cHTML)
       fHtml.Puts('      '+cHTML)
    endif

    ////// stream out form tag
    fHtml.Puts('')
    fHtml.Puts('         <Form METHOD="Get" Action="'+this.cCgiUrl+'VdB_CGI.exe">')
    fHtml.Puts('')
    cExeName = this.cOutputFileName+'.EXE'

    ////// put in hidden so CGI shell program knows what exe to call
    fHtml.Puts('            <input type="hidden" name="EXEFILENAME" '+;
                          'value='+cExeName+'>')

    		// stream form controls
    class::StreamDataControls()

          // submit and reset buttons
    cAlign = iif(this.oform.cWizardType = 'Query','center',;
                 iif(this.oForm.AlignDataCenterRadio.value,'center',;
                 'left'))
    fHtml.Puts('            <DIV align="'+cAlign+'"><p></p><p><Input type="Submit" value="Submit" > '+;
                        '<Input type="Reset" value = "Reset"></p></DIV>')
    

    fHtml.Puts('         </Form>')

    fHtml.Puts('')
    cHTML = class::BuildHTMLString(this.oForm.TextBelowDataEntry.Editor1.value)
    if not empty(cHTML)
       fHtml.Puts('      '+cHTML)
    endif

    this.oform.progress1.value = 70

    class::StreamHomeandMail()
    
    this.oForm.progress1.value = 90

    fHtml.Puts('   </blockquote>')
    fHtml.Puts('   </body>')
    fHtml.Puts('</HTML>')
    fHtml.Close()

    this.oform.progress1.value = 100
    this.oform.progress1.value = 0
    this.oform.finishlabel.text = 'Progress...'

    return true
    
    // Initialize URL

    ///////////Method: StreamtitleandLogo ////////////////////////////////
    //////////Purpose: Outputs title and logo image

    function StreamTitleAndLogo

       local cTitle
       cTitle = ltrim(trim(class::BuildHTMLString(this.oForm.FirstPageTitle.Editor1.value)))
            // if title and logo on same line and both exist

       if this.oform.LeftOfTitleRadio.value and ;
          (not empty(this.logoImage)) and ;
          (not empty(this.oForm.firstPageTitle.editor1.value))
          
             // adjust tags to combine graphic and text in same line.
             local cFirst, cLast
             cFirst = ''
             cLast = ''
             if substr(cTitle,1,2) = '<H'
                //save begining of "header"
                cFirst = substr(cTitle,1,at('>',cTitle))
                //remove H tag and H end tag
                cTitle = substr(cTitle,at('>',cTitle)+1)
               
               
             endif
          fHtml.Puts('      '+cFirst+'<img src="'+this.LogoImage+'"> '+ cTitle)
      
       else

          	// Display logo at center or left, depending on radiobuttons
          if not empty(this.logoImage)
             cHtml = '         <p align="'
             cHtml+= iif(this.oform.TopCenterRadio.value,'center','left')
             cHtml+= '">'
             cHtml+='<img src="'+this.LogoImage+'">'
             cHtml+= '</p>'

             fHtml.Puts(cHtml)
                 // start a new line
          endif
          
            // Display title of page, if there is one
          if not empty(cTitle)
             fHtml.Puts('         '+cTitle)             
          endif


       endif

       return true



    ///////////Method:  CopyGraphicFile ///////////////////////////////
    ///////////Purpose: Copies graphic files from source folder to ////
    ///////////         HTML folder ///////////////////////////////////

    Function CopyGraphicFile(cSource,cDest)	

       local cSourcePath, cFileName, cDestPath, cSaveSource 

       ///// Define destination path
       if not empty(cDest)
          cDestPath = cDest
       else
       		//Define destination path
          cDestPath = this.cHtmlFolder
       endif

       
       ////// If no file specified
       if len(cSource) = 0
          ////// Forget it
          return ''
       endif

       ////// Trim up source path
       cSourcePath = trim(ltrim(cSource))
       
       ////// hard wire default path!
       if at('\',cSourcePath) = 0
          cSourcePath = trim(this.oForm.cLoadPath)+cSourcePath
       endif

       ////// Strip out path to extract filename
       cFileName = trim(cSourcePath)

       if at('\',cFileName) > 0
          cFileName = substr(cFileName,rat('\',cFileName)+1)
       endif

       cDestPath = cDestPath+cFileName


       ////// if Nothing to copy?
       If not file(cSourcePath)
          ////// just forget it
          return ''
       endif 

       ////// This is a workaround of the "File Exists... Overwrite"
       ////// automatic dialog in order to do a "copy all"

       ////// if file already exists
       if file(cDestPath)   

          ////// and they didn't already say 'Yes to all"
          if not this.bCopyAll

             ////// if they say "dont' overwrite"
             if not class::OverwriteFile(cDestPath)
                		//forget it
                return cFile
             else
             			//otherwise, delete the file so no
                     //"overwrite" msgbox() will appear
                dele file (cDestPath)
             endif

          else
                 // if it is already "copyall"
                 // delete the file before copying
             dele file (cDestPath)

          endif
       endif

       	// Try to copy file to HTML folder
       try

          Copy File (cSourcePath) to (cDestPath)

       catch (Exception e)

          msgbox('Cannot copy file '+cSourcePath+' to '+cDestPath+;
                 chr(13)+e.message,'Copy Error',16)
       endtry

       
       Return cFileName  // return the filename


    ////////////Method: BuildBodyString ////////////////////////////////
    ////////////Purpose: Builds string for HTML body attributes ////////
    function BuildBodyString


       local cBody
       cBody = '<body '
       if not empty(this.oForm.BackgroundColorField.value)
          cBody += 'bgcolor="'+;
                    class::HTMLColor(this.oForm.BackgroundcolorField.value)+;
                    '"'
       endif
       if not empty(this.BackGroundImage)
          cBody += ' background="'+this.BackgroundImage+'"'
       endif
       cBody +='>'
       return cBody


    /////////// Method:  HTMLColor ////////////////////////////////////
    /////////// Purpose: converts VdB 7 Color to HTML Color ///////////
    //////////           ex: 0x010203 to #030201 //////////////////////
    /////////// Param:   cColorStr - color expression /////////////////

    function HTMLColor(cColorStr)
       if substr(cColorStr,1,2) = '0x'
          return '#'+substr(cColorStr,7,2)+;
                     substr(cColorStr,5,2)+;
                     substr(cColorStr,3,2)
       endif
       return cColorStr

       
    ///////////Method: Stream Data Controls ///////////////////////////
    ///////////Purpose: Streams out controls for query and data-entry
    
    Function StreamDataControls 

       ////// If this is a query page, stream query "get" controls

       if this.oForm.cWizardType = 'Query'

          ////// if it's a combobox (select) with options
          fHtml.Puts('')

          cHtml  =    '            <Center><P><P>'

          ////// loop through and build option tags
          if this.oForm.ComboboxRadio.value
    
             cHtml += '<Select name="SEARCH" size="1">'
             for n = 1 to alen(this.oform.aOptions,1)
                cOption = this.oForm.aOptions[n]
                cHtml+= '            <Option value="'+cOption+'">'+cOption+;
                         '</Option>'
             next
             cHtml+='</Select></P></P></DIV>'
      
          else  // if it's a text field

             ////// stream text input tags  
             cHtml =  '            <DIV align="center"><P><P>'
             cHtml += '<Input type="Text" name="SEARCH" size="25">'
             cHtml += '</P></P></DIV>'
 
      
          endif
          fHtml.Puts(cHtml)
          fHtml.Puts('')

          return true

       endif
       		//////////////////End of Query Wizard data object streaming

       		//////////////////The following is for Data-Entry Wizard
  
       ////// Check if there are any selected fields!
       if alen(this.oForm.aSelectedFields,1) = 0
          return false      //if not, forget it
       endif

       ////// Initialize data-source properties (for datamodule or table)
       this.cSource = trim(this.oForm.DataSourceField.value)
       this.cFile = iif(at('\',this.cSource) > 0,substr(this.cSource,;
                    rat('\',this.cSource)+1),this.cSource)
                                       
       ////// number of field objects to be streamed
       nNumFields = alen(this.oForm.aSelectedFields,1)
                   
     													

       /////////////////// open query or table

       ////// If user specified a data module as datasource
       if at('.DMD',this.cFile) > 0

          this.cDataType = 'Data Module'
      
          ////// open datamodule source file using low-level File I/O          
          fDmd = new File()

          ////// try to open .dmd file
          try
             fDmd.Open(this.cSource,"R")

          catch (Exception e)
          	 ////// if this fails, error and return
             msgbox('Unable to open disk file '+this.cSource)
             return false
          endtry

 
          ////// Extract .DMD data module class name
          cDmd = ''
          cStr = upper(fDmd.Gets())   // get first line from file

          ////// loop through source looking for class statement
          do 

             if at('CLASS',cStr) > 0 and at('OF DATAMODULE',cStr) > 0
                cDmd = trim(ltrim(substr(cStr,at('CLASS',cStr)+5)))
                cDmd = trim(ltrim(substr(cDmd,1,at('OF DATAMODULE',cDmd)-1)))                    
                exit
             endif

             cStr = upper(fDmd.gets())  // get next line
          
          until fDmd.Eof()              // until end of file

          ////// Clean up
          fDmd.Close()    
          fDmd = ''

          ////// If no class declaration found, sorry...
          if len(trim(cDmd)) = 0        
             msgbox('Cannot find Data Module class in '+this.cSource)
             return false
          endif

          ////// try to instantiate data module          	
          try

             this.dMod = new &cDmd.()  

          catch (Exception e)

             ////// if fail, sorry...
             msgbox('Unable to open Data Module '+this.cSource+chr(13);
                    +e.message)
             return false

          endtry

       ////// if user specified a table as datasource
       else

          this.cDataType = 'Table'

          ////// try to open table

          try

             this.q = new Query()
             this.q.SQL = 'Select * from "'+this.cSource+'"'
             this.q.active = true

          catch (Exception e)

             msgbox('Unable to open table '+this.cSource,'Open Error',16)
             return false

          endtry 

       endif  // endif .dmd or table

       ////////////////////////Stream table controls

       ////// An HTML table is used for alignment purposes

       ////// Store table alignment for shorthand
       cAlign = iif(this.oForm.AlignDataCenterRadio.value,'center','left')

       fHtml.Puts('            <DIV align="'+cAlign+'">')
       fHtml.Puts('            <Table border="0"'+;
                  ' cellpadding="4" cellspacing="0">')

       ////// for manipulation purposes, store referece to form's 
       ////// selected fields array in aSelect
       ////// sSelect preserves the user's field order
       aSelect = this.oForm.aSelectedFields

       for n = 1 to alen(aSelect,1)
          
          //////parse out query/name pairs of array element
          cLookup       = ltrim(trim(aSelect[n]))
          cQuery 			= substr(aSelect[n],1,at('->',aSelect[n])-1)
          cName  			= trim(substr(aSelect[n],at('->',aSelect[n])+2))

          //////get the type of control needed
          //////as well as length and decimals

          if this.cDataType = 'Table'
             this.q.rowset.first()
             cType      = this.q.rowset.fields[cName].type
             nLen       = this.q.rowset.fields[cName].length
             nDecs      = this.q.rowset.fields[cName].decimalLength
          else
             cType 		= 'this.dMod.'+cQuery+'.rowset.fields["'+;
                           cName+'"].type'
             cType 		= &cType.
             cLen       = 'this.dMod.'+cQuery+'.rowset.fields["'+;
                           cName+'"].length'
             nLen       = &cLen.             
             cDecs      = 'this.dMod.'+cQuery+'.rowset.fields["'+;
                           cName+'"].decimalLength'
             nDecs      = &cDecs
          endif

          cType = upper(cType)

          //////String length for HTML Size attribute
          cLen       = ltrim(trim(str(nlen,4)))

          ////// Look up user-defined label for this item (if any)
          ////// in query Wiztemp1

          cLabel = ''
          
          this.oForm.Wiztemp1.Rowset.Applylocate("Field = '"+cLookup+"'")
          if .not. this.oForm.Wiztemp1.Rowset.EndOfset
             cLabel = trim(this.oForm.Wiztemp1.Rowset.Fields['Label'].value)
          endif

          /////// Build pair with '*@' as separator
          /*Note: this separator is used because it is preserved when
                  string is sent to environment by Web server
          */
          xName = cQuery+'*@'+cName

          fHtml.Puts('            <tr>')                    // add table row  

          fHtml.Puts('            <td>'+cLabel+'</td>')  // Add label cell contents

          cHtml =         '            <td>'                           // add control cell
 
          if cType = 'MEMO'
             cHtml+='<textarea rows="2" cols="30" name= "'+xName+'"></textarea>'            
          elseif cType 	= 'LOG'
             cHtml+='<Input type="checkbox" value="OFF" name="'+xName+'">' 
          else        
             cHtml+='<Input type="text" size="'+cLen+'" name="'+xName+'">'
          endif   

          cHtml+= '</td>'

          fHtml.Puts(cHtml)

          fHtml.Puts('            </tr>')

       next

       ///// End table
       fHtml.Puts('            </Table></DIV>')

    	return true


   //////////Method:  StreamHomeAndMail ///////////////////////////////////////
   ////// ///Purpose: Streams out GIFs or text for Home and Mail

   Function StreamHomeAndMail

   local bDoMail, bDoHome, cHTML, bParaDone

   // determine alignment for Home and Mail objects
   cAlign = iif(this.oForm.cWizardType = 'Query' or ;
            (this.oForm.cWizardType = 'Data' and ;
            this.oForm.AlignDataCenterRadio.value),;
            'center','left')

   cHtml = ''
   bParaDone = false

   //////first find out if the addresses and info needed exist
   bDoHome = not( empty(this.oform.HomeImageField.value) or;
                  empty(this.oform.HomeURLField.value) )
   bDoMail = not( empty(this.oform.MailImageField.value) or;
                  empty(this.oform.MailAddressField.value) )

   ////// if home image to be used
   if bDoHome

      ////// if paragraph not already streamed
      if not bParaDone
         cHtml+= '<DIV align="'+cAlign+'"><p></p>'
         bParaDone = true
      endif

      ////// Clean up user-input URL for "Home"
      cUrl = trim(ltrim(this.oForm.HomeUrlField.value))
      if substr(upper(cUrl),1,7) # 'HTTP://'
         cUrl = 'http://'+cUrl
      endif

      ////// if text hyperlink to be used   
      if this.oForm.UseTextRadio.value
         cHtml += ;
               '<a href="'+cUrl+'">'+;
               trim(ltrim(this.oForm.HomeImageField.value))+;
               '</a>'
         ////// if image object to be used for home link
         else
           cHtml += ;
               '<a href="'+cUrl+'">'+;
               '<img src="'+trim(this.oForm.cHomeImage)+'" border="0">'+;
               '</a>'
               
         endif

      endif

      ////// If mail is to be used
      if bDoMail

         ////// if paragraph tag is not yet streamed
         if not bParaDone
            cHtml+= '<DIV align="'+cAlign+'"><p></p>'
            bParaDone = true
         endif

         ////// if text HTML link is to be used
         if this.oForm.UseTextRadio.value
           cHtml += ;
               '&nbsp<a href="mailto:'+trim(ltrim(this.oform.MailAddressField.value))+'">'+;
               trim(ltrim(this.oForm.MailImageField.value))+;
               '</a>'

         ////// If image object is to be used for "home" link
         else
           cHtml += ;
               '&nbsp; &nbsp; <a href="mailto:'+trim(ltrim(this.oform.MailAddressField.value))+'">'+;
               '<img src="'+trim(this.Mailimage)+'" border="0">'+;
               '</a>'
               
         endif
      endif

      ////// if paragraph tag has been done, close it out
      if bParaDone
         cHtml += '</DIV>'
         fHtml.Puts('      '+cHtml)
      endif

      return cHtml



   //////////////// Method:   CleanUpFolder ///////////////////////////
   //////////////// Purpose:Returns path trimmed with trailing backslash

   function CleanUpFolder(cPath)
      cPath = ltrim(trim(cPath))
      if substr(cPath,len(cPath),1) # '\'
         cPath +='\'
      endif
      
      return cPath


   ///////////////Method: BuildHTMLString////////////////////////////
   ///////////////Purpose: Substitutes HTML macros for quotation marks
   

   /////////  Method:  OverwriteFile /////////////////////////////////
   /////////  Purpose: Special msgbox() form for overwriting existing 
   /////////           files with a "yes to all" /////////////////////

   Function OverWriteFile(cFileName)
      this.bOverWrite = false


      this.oF = new form()
      with (this.oF)
         scaleFontBold = false
         height = 4.5909
         left = 26.7143
         autocenter = true
         top = 3
         width = 49.2857
         mdi = false
         text = "Overwrite"
      endwith
      this.oF.oParent = this

      this.oF.IMAGE1 = new IMAGE(this.oF)
      with (this.oF.IMAGE1)
         height = 1.5455
         left = 0.2857
         top = 0.3182
         width = 5.7143
         metric = 0	// Chars
         dataSource = "FILENAME Question.bmp"
         alignment = 4	// True Size
         borderStyle = 3	// None
      endwith


      this.oF.TEXT1 = new TEXT(this.oF)
         with (this.oF.TEXT1)
         height = 1.8182
         left = 8.1429
         top = 0.7273
         width = 37.1429
         metric = 0	// Chars
         colorNormal = "BtnText/BtnFace"
         fontSize = 8
         fontName = 'MS Sans Serif'        
         text = '<p align="Center">'
         text+= 'File: '+trim(cFileName)+' already exists! Overwrite?'
         text+= '</p>'
      endwith


      this.oF.YESBUTTON = new PUSHBUTTON(this.oF)
      with (this.oF.YESBUTTON)
         onClick = {;form.oParent.bOverWrite = true;form.close()}
         height = 0.9545
         left = 7.5714
         top = 3.1364
         width = 10.7143
         text = "&Yes"
         metric = 0	// Chars
         fontSize = 8
         group = false
         colorNormal = "BtnText/BtnFace"
         value = false
      endwith


      this.oF.YESTOALLBUTTON = new PUSHBUTTON(this.oF)
      with (this.oF.YESTOALLBUTTON)
         onClick = {;form.oParent.bOverWrite = true;form.oParent.bCopyAll=true;form.close()}
         height = 0.9545
         left = 19.2857
         top = 3.1364
         width = 10.7143
         text = "Yes to &All"
         metric = 0	// Chars
         fontSize = 8
         group = false
         colorNormal = "BtnText/BtnFace"
         value = false
      endwith

      this.oF.NOBUTTON = new PUSHBUTTON(this.oF)
      with (this.oF.NOBUTTON)
         onClick = {;form.close()}
         height = 0.9545
         left = 31
         top = 3.1364
         width = 10.7143
         text = "&No"
         metric = 0	// Chars
         fontSize = 8
         group = false
         colorNormal = "BtnText/BtnFace"
         value = false
      endwith

      ?? chr(7)
      this.oF.ReadModal()
      this.oF = Null  // release modal dialog's reference
      return this.bOverWrite



   ///////////Method: 	StreamCGI  ///////////////////////////////////////////
   ///////////Purpose:  The other main method of class Out ///////////////////
   ///////////          this method streams .prgs and and ////////////////////
   ///////////          compiles and generates an .EXE  //////////////////////

   function StreamCGI(nRecursionLevel)
      /* Note: recursion level is used for drill-down reports
               Level 1 generates first .exe
               Level 2 generates second .exe
      */
      nRecursionLevel = iif(empty(nRecursionLevel),1,nRecursionLevel)

      ////// properties used for drill-down reports only 
      this.cLinkRef   = ''
      this.cLinkExpr  = ''

      ///////////////// don't do it if it's a static publish
      ///////////////// all others stream CGI

      if this.oForm.cWizardType = 'Publish' and ;
         not this.oForm.LiveReportRadio.value
         return true
      endif

      ////// Update progress bar label
      this.oForm.Finishlabel.text = 'Streaming CGI code...'
      this.oForm.Finishlabel.visible  = false // workaround for display prob.
      this.oForm.Finishlabel.visible = true
  

      Local n
      this.cClass = ''
      this.cDataType = 'Table'

      this.oForm.Finishlabel.Text = 'Streaming CGI...'
      this.oForm.progress1.value = 10
   
      ///////////////////Data-Entry Wizard setup /////////////////////////
      if this.oForm.cWizardType = 'Data Entry'

         ///// Paste up .prg path 
         cPrgName = trim(this.oForm.FileNameField.value)
         cPrgPath = this.cCGIFolder+cPrgName+'.prg'

         ////// this is for compatibility with the other wizards
         this.cReportName = cPrgName
         this.cReportPath = cPrgPath

         ////// Create an array for compile and build
         this.aObj = new array()
         this.aObj.add(cPrgPath+iif(nRecursionLevel=2,'2',''))
         
   

         ////// Extract the datamodule name from the source file
         cQueryName = ''   
         cDmodName = ''
         cQueryName = ''
         this.cDataPath = trim(this.oForm.dataSourceField.value)

         if upper(right(this.cDataPath,4)) = '.DMD'
            this.cDataType = 'Datamodule'
            this.aObj.add(this.cDataPath)

            ////// try to open the .dmd file
            try
               fDmd = new File()
               fDmd.Open(trim(this.cDataPath),"R")
            catch (Exception e)
               msgbox(e.message,'Data Module Open Error',16)
               return false
            endtry

            do

               cStr = trim(fDmd.Gets(this.nHnd))

               ////// if class declaration is found
               if at('CLASS ',upper(cStr)) > 0 and at('OF',upper(cStr)) > 0
                                                        // get class name
                  cStr = ltrim(substr(cStr,at('CLASS',upper(cStr))+5))           
                  cDModName = trim(substr(cStr,1,at(' ',cStr)-1))
                  exit
               endif

            until fDmd.Eof()

            fDmd.Close()
            fDmd = ''

            ////// if no data module class name is found, forget it!
            if len(trim(cDmodName)) = 0
                 msgbox('Cannot find class declaration in: '+cDataPath,;
                        'Data Module Error',16)
                 return false
            endif

         else  // if it's a table

            this.cDataType = 'Table'

            ////// get table name to use as query name
            cQueryName = substr(this.cDataPath,1, rat('.',this.cDataPath)-1)

            if at('\',cQueryName) > 0
               cQueryName = substr(cQueryName,rat('\',this.cDataPath)+1)
            endif
        
         endif


      ////////// if this is a live publish or query
      else

         if nRecursionLevel = 2  // if second drill-down report

            ////// Extract Report 2 path
            this.cReportPath = trim(this.oForm.SecondReportField.value)
            if len(trim(this.cReportPath)) = 0
               return false
            endif

            ////// Extract Report 2 Name
            if at('\',this.cReportPath) > 0
               this.cReportName = substr(this.cReportPath,;
                  rat('\',this.cReportPath)+1)
            else
                this.cReportName = this.cReportPath

            endif


            if at('.',this.cReportName) > 0
               this.cReportName = substr(this.cReportName,1,;
                    at('.',this.cReportName)-1)

            endif
         endif

       

         if this.oForm.cWizardType = 'Query'
            ///// Paste up .prg path 
            cPrgName = trim(this.oForm.FileNameField.value)+;
                       iif(nRecursionLevel=2,'2','')
            cPrgPath = this.cCGIFolder+cPrgName+'.prg'

         else
            /////// Paste up .prg path 
            cPrgPath = this.cCGIFolder+This.cReportName+'.prg'
            cPrgName = this.cReportName+'.prg'

         endif


         ////// Create an array for compile and build
         this.aObj = new array()
         this.aObj.add(cPrgPath)
         this.aObj.add(this.cReportPath)

         this.oForm.progress1.value = 10  // update progress bar

         ////// Try to open the report file and extract various statements
         fRep = new File()
      
         try
            fRep.Open(this.cReportPath,'R')

         catch (Exception e)
            msgbox(e.message,'Fail Report Open'+chr(13)+e.message,16)
            return false

         endtry


         ////// Parse out file for class, set proc and linking text field

         ////// declare and init vars for report source parsing routines
         private cCurrentTextRef, cLinkField
         cCurrentTextRef = ''
         cLinkField = ''
         this.cLinkRef = ''
         this.cLinkExpr = ''

         do
           
            cStr = upper(fRep.Gets())

            ////// parse out each line for a set proc statement
            if at('SET PROC',cStr) > 0
               ////// if found, get the path and add it to the array
               cProc = trim( ltrim( substr(cStr,at(' TO ',cStr)+3) ) )

               ////// remove any "additive" statements
               if at('ADDI',upper(cProc))
                  cProc  = ltrim(trim(substr(cProc,1,at('ADDI',cProc)-1)))
               endif
               this.aObj.add(cProc)
            endif


            ////// parse out each line for the class declaration 
            if at('CLASS',cStr) > 0 and at('REPORT',cStr) >0 ;
               and at('OF',cStr) > 0
               this.cClass = trim( ltrim(  substr(cStr,at('CLASS',cStr)+5) ) )
               this.cClass = trim( ltrim(  substr(this.cClass,1, at(' OF ',this.cClass)-1) ) )
            endif

             
            ////// If this is first of two drill-down reports, 
            ////// and the link expression and reference have not yet been
            ////// set, parse out each line for the linking text object



            if len(trim(this.cLinkRef)) = 0 and ;
               nRecursionLevel = 1 and ;
               this.oForm.DrillDownRadio.value
                 
               /////// if this is a text control, extract object reference
               /////// just in case it's the one selected by user for linking
               /////// the two reports together
               if at('NEW TEXT(',cStr) > 0
                  cCurrentTextRef = ltrim( trim( substr(cStr,1, at('=',cStr)-1) ))
                  if at('THIS.',cCurrentTextRef) > 0
                     cCurrentTextRef = substr(cCurrentTextRef,;
                                      at('THIS.',cCurrentTextRef)+5)
                  endif
               endif


                /////// If this is the text field of a text control
               if at("TEXT",cStr) > 0 and at('FIELDS[',cStr) > 0  and;
                  at('{||',cStr) > 0       
                  cLinkField = trim(ltrim(cStr.Substring(at('FIELDS[',cStr)+7,;
                                                 at(']',cStr)-1)))
                  ////// strip off quotes (needed in case of spaces between
                  ////// square brackets and quotes)
                  if substr(cLinkField,1,1) = '"'
                     cLinkfield = substr(cLinkField,2)
                  endif
                  if substr(cLinkField,len(cLinkField),1) = '"'
                     cLinkfield = substr(cLinkField,1,;
                                  len(cLinkField)-1)
                  endif

               

                  ////// if this text object matches the one the user
                  ////// selected to link the two reports together
                  if (not empty(this.oForm.SecondReportFieldList.value)) and ;
                     upper(cLinkField) = ;
                     upper( trim( ltrim( this.oForm.SecondReportFieldList.value )))
                     
                     //clean up text expression and remove codeblock syntax
                     this.cLinkExpr = trim( ltrim( substr(cStr,at('=',cStr)+1) ) )
                     if substr(this.cLinkExpr,1,3) = '{||'
                        this.cLinkExpr = substr(this.cLinkExpr,4)
                     endif
                     if substr(this.cLinkExpr, len(this.cLinkExpr),1) = '}'
                        this.cLinkExpr = substr(this.cLinkExpr,1,len(this.cLinkExpr)-1)
                     endif

                     this.cLinkRef   = cCurrentTextRef

                  endif
               endif 

            endif

         until fRep.Eof() // until done with file

         this.oform.progress1.value = 25

         ////// clean up
         fRep.close()
         frep = Null


      endif  // data-entry or live publish/query wizard

      ////////// The following code is shared between all Wizard types


      ////// Copy Trans.gif - required for reports
      if not file(this.cCgiFolder+'Trans.gif')
         try

            copy file(this.oForm.cLoadPath+'Trans.gif') to ;
                 (this.cCGIFolder+'Trans.gif')

         catch (Exception e)

            msgbox('Unable to copy Trans.gif'+chr(13)+e.Message,;
                   'Copy Failed',16)
            return false

         endtry

      endif


      ////// Copy Vdb_CGI.exe if needed
      if not file(this.cCgiFolder+'VdB_CGI.exe')
         if not file(this.oForm.cLoadPath+'Vdb_Cgi.exe')
            msgbox('Cannot find '+this.cCgiFolder+'VdB_Cgi.exe.','Missing File',16)
            return false
         endif

         copy file(this.oForm.cLoadPath+'VdB_Cgi.exe') to ;
                  (this.cCgiFolder+'VdB_Cgi.exe')
      endif


      ////// Copy VdbFast.exe if needed
      if not file(this.cCgiFolder+'VdBFast.exe')
         if not file(this.oForm.cLoadPath+'VdBFast.exe')
            msgbox('Cannot find '+this.cCgiFolder+'VdBFast.exe.','Missing File',16)
            return false
         endif

         copy file(this.oForm.cLoadPath+'VdBFast.exe') to ;
                  (this.cCgiFolder+'VdBFast.exe')
      endif


      ///// Create new .prg file

      if file(cPrgPath)
          dele file (cPrgPath)
      endif
 

      fPrg = new File()
      fPrg.Create(cPrgPath,'RA')

      this.oForm.progress1.value = 40

      n = len(cPrgName)+7  // to indent, for appearance only

      ////// header comments
      fPrg.Puts('/* '+cPrgName+' - Program file for server-side CGI execution')
      fPrg.Puts(space(n)+'Generated by the Visual dBASE 7 '+;
                          this.oForm.cWizardType+' Web Wizard')
      fPrg.Puts(space(n)+'Date: '+dtoc(date())+' Time: '+time())
      fPrg.Puts(' ')
      fPrg.Puts('   Note: '+space(n-9)+'Param:fOutFile is the CGI Reponse file name.')
      fPrg.Puts(space(n)+'It is created and opened by VdB_Cgi before calling')
      fPrg.Puts(space(n)+'this application.')
      fPrg.Puts('*/')
      fPrg.Puts(' ')
     
      ////// if Query application, parse out the query string
      if this.oForm.cWizardType = 'Query'

         fPrg.Puts('Param fOutFile, cQuery')
         fPrg.Puts(' ')
         fPrg.Puts('////// Create query variable for response reports to use')
         fPrg.Puts('Public cSearch')
         fPrg.Puts('cSearch = " "')
         fPrg.Puts(' ')

         
         ////// if first drill-down report or stand-alone single-report 
         ////// query, search string comes from HTML control
         if nRecursionLevel = 1

            ////// Query string must be gotten from environment
            /*     note: on a Query, there should only be two name/value
                         pairs returned:
                         EXEFILENAME=   and
                         <Query Controlname>= (search string)
            */
            fPrg.Puts('   ////// get query string from environment')
            fPrg.Puts('   cEnv = trim(GetEnv("QUERY_STRING"))')
            fPrg.Puts(' ')
            fPrg.Puts('   ////// EXEFILENAME is always the first pair')
            fPrg.Puts('   ////// and "&" is used as delimiter in CGI,') 
            fPrg.Puts('   ////// so look for second name/value pair')
            fPrg.Puts(' ')
            fPrg.Puts('   if at("&",cEnv) > 0')
            fPrg.Puts('      cEnv = substr(cEnv,at("&",cEnv)+1)')
            fPrg.Puts('      cSearch = OEMFormat(substr(cEnv,at("=",cEnv)+1))')
            fPrg.Puts('   endif')
            fPrg.Puts(' ')        
           

         /////// If this is second report on drill-down query
         else

            ////// Query string is sent as second commmand line parameter
            ////// Only for second drill-down report.
            fPrg.Puts(' ')
            fPrg.Puts('////// See if search param is sent from command line')
            fPrg.Puts("if not empty(cQuery)")
            fPrg.Puts("   cSearch = cQuery")
            fPrg.Puts("endif")
            fprg.Puts(" ")

         endif

      elseif this.oForm.cWizardType = 'Data Entry'

         fPrg.Puts('Param fOutFile')
         fPrg.Puts(' ')
         fPrg.Puts('private aData, cStr')
         fPrg.Puts('aData = new array()')
         fPrg.Puts(' ')
         fPrg.Puts('////// Get the Query string from the environment')
         fPrg.Puts('cEnv = getenv("QUERY_STRING")')
         fPrg.Puts('if len(trim(cEnv)) = 0')
         fPrg.Puts('   errorPage("Message: No data received from server!",0)')
         fPrg.Puts('endif')
         fPrg.Puts(' ')
         fPrg.Puts('////// EXEFILENAME is always the first pair')
         fPrg.Puts('////// "&" is used as delimiter in CGI,') 
         fPrg.Puts('//////  so look for second pair')
         fPrg.Puts(' ')
         fPrg.Puts('if at("&",cEnv) > 0')
         fPrg.Puts('   cEnv = substr(cEnv,at("&",cEnv)+1)')
         fPrg.Puts('endif')
         fPrg.Puts(' ')
         fPrg.Puts('do while at("=",cEnv) > 0')
         fPrg.Puts('   /////// CGI uses "&" separators between pairs')
         fPrg.Puts('   if at("&",cEnv) > 0 // if more than one pair left')
         fPrg.Puts('      cData = OEMFormat(substr(cEnv,1,at("&",cEnv)-1))')
         fPrg.Puts('      cEnv = substr(cEnv,at("&",cEnv)+1)')
         fPrg.Puts('      ////// If last pair ')
         fPrg.Puts('   else')
         fPrg.Puts('      cData = OEMFormat(cEnv)    // take the whole string')
         fPrg.Puts('      cEnv = " "')
         fPrg.Puts('   endif')
         fPrg.Puts(' ')
         fPrg.Puts('   if len(trim(cData)) > 0')
         fPrg.Puts('      aData.add(cData)')
         fPrg.Puts('   endif')
         fPrg.Puts('enddo')
         fPrg.Puts(' ')
         fPrg.Puts('/////// Sort array')
         fPrg.Puts('aData.Sort()')
         fPrg.Puts(' ')
         fPrg.Puts('////// Open Query or Data Module')

         if this.cDataType = 'Datamodule'
           fPrg.Puts('Set Procedure to '+this.cDataPath+' additive')
           fPrg.Puts('dMod = new '+cDModName+'()')
         else
           fPrg.Puts(cQueryName+' = new query()')
           cSql = 'Select * from '+'"'+this.cDataPath+'"'
           fPrg.Puts(cQueryName+'.sql = '+"'"+cSql+"'")
           fPrg.Puts(' ')
           fPrg.Puts('// attempt to activate query')
           fPrg.Puts('try')
           fPrg.Puts('   '+cQueryName+'.active = true')
           fPrg.Puts('catch (exception e)')
           fPrg.Puts('   Errorpage(e.message)')
           fPrg.Puts('endTry')
           fPrg.Puts(" ")
         endif

         fPrg.Puts('////// sort array by query and field')
         fPrg.Puts('private n, cStr, bAppendDone, cQuery, cType, cRow')
         fPrg.Puts('cQuery = ""')
         fPrg.Puts('cType = ""')
         fPrg.Puts('cRow =""')
         fPrg.Puts('n=1')
         fPrg.Puts(' ')
         fPrg.Puts('////// Traverse array of field/value pairs and update')
         fPrg.Puts('////// rowset fields with data')
         fPrg.Puts(' ')
         fPrg.Puts('bAppendDone = false')
         fPrg.Puts(' ')
         fPrg.Puts('Do while n <= alen(aData,1)')
         fPrg.Puts('   ////// store array element for extraction')
         fPrg.Puts('   ////// format: Query*@Field=Data')
         fPrg.Puts('   cStr = trim(aData[n])')
         fPrg.Puts(' ')
         fPrg.Puts('   if at("=",cStr) > 0  // if a valid name/value pair')
         fPrg.Puts('      ////// extract query name')
         fPrg.Puts('      cQuery = substr(cStr,1,at("*@",cStr)-1)')
         fPrg.Puts('      ////// extract field name')
         fPrg.Puts('      cField = substr(cStr,at("*@",cStr)+2)')
         fPrg.Puts('      cField = substr(cField,1,at("=",cField)-1)')
         fPrg.Puts('      ////// extract data')
         fPrg.Puts('      cData  = trim(substr(cStr,at("=",cStr)+1))')
         fPrg.Puts('      ////// find out the type of the field to be updated')
         if this.cDataType = 'Datamodule'
            fPrg.Puts('      cRow = "dMod."+cQuery+".Rowset"')
         else
            fPrg.Puts('      cRow = cQuery+".rowset"')
         endif
         cQuote = "'"+'"'+"'"
         cF = "'.Fields['+"
         ct = "'].type'"
         cStr = 'cType = cRow + '+cF+cQuote+'+cField+'+cQuote+'+"].type"'             
         fPrg.Puts('      '+cStr)
         fPrg.Puts('      cType  = &cType')
         fPrg.Puts(' ')
         fPrg.Puts('      ////// try to append new row')
         fPrg.Puts('      if not bAppendDone')
         fPrg.Puts('         cAppend = cRow+".beginappend()"')
         fPrg.Puts('         try')
         fPrg.Puts('            &cAppend.')
         fPrg.Puts('         catch (Exception e)')
         fPrg.Puts('            errorPage(e.message,e.Lineno)')
         fPrg.Puts('         endtry')
         fPrg.Puts('      endif')
         fPrg.Puts('      bAppendDone = true')
         fPrg.Puts(' ')
         fPrg.Puts('      ////// Set up "update strings" for Macro ')   
         fPrg.Puts('      cUpdate = cRow')
         fPrg.Puts('      cF = '+cQuote+'+cField+'+cQuote)
         fPrg.Puts(' ')
         fPrg.Puts('      if cType 	= "LOGICAL"')
         fPrg.Puts('         cUpdate += ".fields["+cF+"].value = "+iif(cData = "Y",true,false)')
         fPrg.Puts(' ')
         fPrg.Puts('      elseif cType = "NUMERIC" or ;')
         fPrg.Puts('         cType = "INTEGER" or ;')
         fPrg.Puts('         cType = "LONG"')
         fPrg.Puts('         cUpdate +=  ".fields["+cF+"].value = "+val(cData)')
         fPrg.Puts(' ')
         fPrg.Puts('      elseif cType = "DATE"')
         fPrg.Puts('         cUpdate += ".fields["+cF+"].value = "+dtoc(cData)')
         fPrg.Puts(' ')
         fPrg.Puts('      elseif cType = "AUTOINCREMENT"')
         fPrg.Puts('          // do nothing!! Autoincrement is automatic')
         fPrg.Puts(' ')
         fPrg.Puts('      else') 
         fPrg.Puts('         cUpdate += ".fields["+cF+"].value = "+'+cQuote+'+cData+'+cQuote)
         fPrg.Puts(' ')
         fPrg.Puts('      endif')
         fPrg.Puts('   endif')
         fPrg.Puts(' ')
         fPrg.Puts('   if cType # "AUTOINCREMENT"')
         fPrg.Puts('      Try')
         fPrg.Puts('         &cUpdate.  // Update field')
         fPrg.Puts('      Catch (Exception e)')
         fPrg.Puts('         ErrorPage(e.message, e.lineno)')
         fPrg.Puts('      endtry')
         fPrg.Puts('   endif')
         fPrg.Puts(' ')
         fPrg.Puts('   ++n')
         fPrg.Puts('   Local bSaveData')
         fPrg.Puts('   bSaveData = false')
         fPrg.Puts('   ////// if element beyond last one in array')
         fPrg.Puts(' ')
         fPrg.Puts('   if n > alen(aData,1)')
         fPrg.Puts('      bSaveData = true')
         fPrg.Puts(' ')
         fPrg.Puts('   else')
         fPrg.Puts('      ////// determine if data is for a new query')
         fPrg.Puts('      cStr = aData[n]')
         fPrg.Puts('      ////// find out what query the next element uses')
         fPrg.Puts('      cNewQuery = substr(cStr,1,at("*@",cStr)-1)')
         fPrg.Puts("      ////// if it's different than previous, save current row")
         fPrg.Puts('      bSaveData = ( cNewQuery # cQuery )')
         fPrg.Puts(' ')
         fPrg.Puts('   endif')
         fPrg.Puts(' ')
         fPrg.Puts('   if bAppendDone and bSaveData')
         fPrg.Puts('      cSave = cRow+".Save()"')
         fPrg.Puts('      &cSave.')
         fPrg.Puts('      bAppendDone = false  // reset for next query')
         fPrg.Puts(' ')
         fPrg.Puts('   endif')
         fPrg.Puts(' ')
         fPrg.Puts('enddo')
         fPrg.Puts(' ')

         fPrg.Puts('GenerateResponse(fOutFile)  // return response page to Browser')
         fPrg.Puts('                    // either response: error or thank you, quits.')
         fPrg.Puts(' ')
         fPrg.Puts('   Quit')
         fPrg.Puts(' ')
         fPrg.Puts('////////////////function GenerateResponse//////////////')
         fPrg.Puts('////////////////outputs "thank you" to CGI ////////////')
         fPrg.Puts(' ')
         fPrg.Puts('function GenerateResponse(fOutFile)')
         fPrg.Puts('   fOut = new File()')
         fPrg.Puts('   If not file(fOutFile)')
         fPrg.Puts('      fOut.Create(fOutFile,"RA")')
         fPrg.Puts('   else')
         fPrg.Puts('      fOut.Open(fOutFile,"RA")')
         fPrg.Puts('   Endif')
         fPrg.Puts(' ')
      //   fPrg.Puts('   fOut.Puts("Content-type: text/html")')
         fPrg.Puts('   ')                      && fOut.Puts(" ")')
         fPrg.Puts("   fOut.Puts('<HTML>')")   && ab hier deutsche Texte
         fPrg.Puts("   fOut.Puts('<TITLE>Daten eingetragen</TITLE>')")
         this.BackgroundImage= trim(this.oForm.BackGroundImageField.value)
         fPrg.Puts('   fOut.Puts('+"'"+class::BuildBodyString()+"'"+')')
         fPrg.Puts("   fOut.Puts('<blockquote>')")
         cHtml = this.oForm.TextOnResponsePage.editor1.value

         if len(trim(cHtml)) = 0
            cHtml = '<H1>Die Daten wurden eingetragen!</H1>'
         endif
         fPrg.Puts("   fOut.Puts('"+cHtml+"')")
         fPrg.Puts("   fOut.Puts('</Blockquote>')")
         fPrg.Puts("   fOut.Puts('</body>')")
         fPrg.Puts("   fOut.Puts('</HTML>')")
         fPrg.Puts('   ')
         fPrg.Puts('   fOut.close()')
      elseif this.oForm.cWizardType = 'Publish'	and this.oForm.LiveReportRadio.value && zusaetzlicher Fall
         fPrg.Puts('Param fOutFile')						&& Fehlerbereinigung
      endif // Data-Entry wizard           

      if this.oForm.cWizardType # 'Data Entry'
         fPrg.Puts('')
         fPrg.Puts('   ////// Set procedure to report file')
         fPrg.Puts('')
         fPrg.Puts('      Set procedure to '+this.cReportPath+' additive')
         fPrg.Puts('')
         fPrg.Puts('      ////// Instantiate report and set properties for HTML output')
         fPrg.Puts('')
         fPrg.Puts('   try')
         fPrg.Puts('')
         fPrg.Puts('      r = new '+this.cClass+'()')
         fPrg.Puts('')
         fPrg.Puts('   catch (exception e)')
         fPrg.Puts('')
         fPrg.Puts('      errorPage(e.message)')
         fPrg.Puts('')
         fPrg.Puts('   endtry')
         fPrg.Puts('')
         fPrg.Puts('   r.Output = 4 // HTML')
         fPrg.Puts('   r.OutputFileName = fOutFile')
         fPrg.Puts('')

         // reset text property of text field that links the two reports
         //      &&text= {||'<A href=http://www.ksoftinc.com/cgi/vdb_cgi.exe?Y.EXE~'+;
                 this.form.alan1.rowset.fields["name"].value+'>'+this.form.alan1.rowset.fields["name"].value+'</A>'}

         if nRecursionLevel = 1 and this.oForm.DrilldownRadio.value

           cStr = '{||"<A href='+this.cCgiUrl+'Vdb_Cgi.exe?'+;
                   this.cOutputfilename+'2.EXE~"+'+this.cLinkExpr+'+">"'+;
                   '+'+this.cLinkExpr+'+'+'"</A>"}'
           fPrg.Puts('   r.'+this.cLinkRef+'.text = '+cStr)
           fPrg.Puts('')

         endif

         fPrg.Puts('   ////// Render report to the CGI Response file')
         fPrg.Puts('   try')
         fPrg.Puts('      r.render()')
         fPrg.Puts('   catch (Exception e)')
         fPrg.Puts('      ErrorPage(e.message)')
         fPrg.Puts('   endtry')
         fPrg.Puts(' ')
         fPrg.Puts('   ////// Clean up')
         fPrg.Puts('   r = Null')
         fPrg.Puts('   close proc '+this.cReportPath)
         fPrg.Puts('')
        
         fPrg.Puts('   ////// all done')
         fPrg.Puts('   Quit')
         fPrg.Puts('')
         fPrg.Puts('')

      endif

      // shared by all Wizards ////////////////////////////////////////////
      fPrg.Puts('////// Function:   ErrorPage //////////////////////////)')
      fPrg.Puts('////// Purpose:    Returns error page if App fails //////')
      fPrg.Puts('////// Param:      cMess  = Error Message //////////////')
      fPrg.Puts('')
      fPrg.Puts('function ErrorPage(cMess,nLineNo)')
      fPrg.Puts('   ////// Create new response file') 
      fPrg.Puts('   nLineNo = iif(empty(nLineNo),0,nLineNo)')
      fPrg.Puts('')
      fPrg.Puts('   fOut = new file()')
      fPrg.Puts('   fOut.Open(fOutFile,"RA")')
      fPrg.Puts('   fOut.Puts("")')
      fPrg.Puts('')
      fPrg.Puts('   ////// Generate HTML error page')
      fPrg.Puts('   fOut.Puts(" ")')
      fPrg.Puts('   fOut.Puts("<HTML>")')                   && deutsche Texte
      fPrg.Puts('   fOut.Puts("<HEAD><TITLE>Visual-dBASE 7 Web-Wizard Fehler-Seite</TITLE>")')
      fPrg.Puts('   fOut.Puts("<BODY>")')
      fPrg.Puts('   fOut.Puts("")')                    
      fPrg.Puts('   fOut.Puts("   <H2>Sorry, ein Fehler trat auf dem Server auf!</H2>")')
      fPrg.Puts('   fOut.Puts("")')
      fPrg.Puts('   fOut.Puts("   <P>Fehler: "+cMess+" </P>")')
      fPrg.Puts('   fOut.Puts("   <P>Zeile Nr: "+ltrim(str(nLineNo,5))+"</P>")')
      fPrg.Puts('   fOut.Puts("   <P>Bet&auml;tigen Sie den <I>Zur&uuml;ck</I>-Schalter Ihres Browsers und versuchen Sie erneut '+;
                    'oder kontaktieren Sie den Web-Master.")')
      fPrg.Puts('   fOut.Puts("")')
      fPrg.Puts('   fOut.Puts("</BODY>")')
      fPrg.Puts('   fOut.Puts("</HTML>")')
      fPrg.Puts('   fOut.close()')
      fPrg.Puts('   Quit')
      fPrg.Puts(' ')
      fPrg.Puts('///////////// Function OEMFormat //////////////////////////////')
      fPrg.Puts('///////////// Purpose - transforms CGI escape chars to OEM/////')
      fPrg.Puts('///////////// Param: cStr - Environment Variable Value ////////')
      fPrg.Puts(' ')                                        && hier geht die neue Variante los
      fPrg.Puts('function OEMFormat(cStr)                   && verbesserte Version')
      fPrg.Puts('   Private p                               && Positionszeiger')

      fPrg.Puts('   ////// remove terminating character     && Wozu das wohl gut ist ???')
      fPrg.Puts('   if at("%21",cStr) > 0')
      fPrg.Puts('      cStr = substr(cStr,1,at("%21",cStr)-1)')
      fPrg.Puts('   endif')

      fPrg.Puts('   ////// remove "Pluses" used as space keepers in CGI')
      fPrg.Puts('   do while at("+",cStr) > 0')
      fPrg.Puts('      cStr = stuff(cStr,at("+",cStr),1," ")')
      fPrg.Puts('   enddo')

      fPrg.Puts('   ////// add in escape characters convert to normal Characters')
      fPrg.Puts('   do while at("%",cStr) > 0')
      fPrg.Puts('    p = at("%",cStr)')
      fPrg.Puts('    cStr = stuff(cStr,p,3,OEM(chr(HToI(substr(cStr,p+1,2)))))')
      fPrg.Puts('   enddo               && alles ausser "%" selbst ist umkodiert')


      && hier stand frueher der Rest des uneffektiven Formatierungscode












      && Die Luecke soll die originale Zeilennumerierung beibehalten

      fPrg.Puts('return cStr')
      fPrg.Close() 
      fPrg = null

      
      this.oForm.Finishlabel.text = 'Recompiling all files...'
      this.oForm.progress1.value = 80 // update progress bar and label

      ////// re-compile all files
      for n =1 to alen(this.aObj,1)

         try

            if not empty(this.aObj[n])
               compile (this.aObj[n])
            endif

         catch (Exception e)

            msgbox(e.message,'Compile Failed: '+this.aObj[n],16)
            return false

         endtry

      next

      this.oForm.Finishlabel.text = 'Building .exe...'
      this.oForm.progress1.value = 90 // Update progress bar and label

      ////// Build a "build string"
      private cFile,cBuild


      cBuild = 'Build "'+substr(this.aObj[1],1, len(this.aObj[1])-1)+'o"'


      for n = 2 to alen(this.aObj,1)
          cFile = this.aObj[n]
          ////// paste the object file name
          cFile = '"'+substr(cFile,1,len(cFile)-1)+'o'+'"'
          cBuild += ' ,'+cFile
      next

      ////// Build the .exe file name for Web application

      private cExePath

      if nRecursionLevel = 1
         cExePath = this.cCgiFolder+this.cOutputFileName+'.exe'
      else
         cExePath = this.cCgiFolder+this.cOutPutFileName+'2.exe'
      endif

      if at(' ',cExePath) > 0
         cBuild += ' To "'+trim(ltrim(cExePath))+'"'
      else
         cBuild += ' To '+cExePath
      endif

      try

        &cBuild.

      catch (Exception e)     

        msgbox(substr(e.message,1,60)+chr(13)+substr(e.message,61),'EXE Build Error',16)
        return false

      endtry
 
   
   return true


   //////////// Method: BuildHTMLString ////////////////////////////////
   //////////// Purpose: Replaces quotes with HTML macros //////////////
  
   function BuildHTMLString(cStr)
      //// extract text-only portion from HTML expression
      //// get "front" tags
      //// get "back" tags
      
      do while at("'",cStr) > 0
 
         cStr = substr(cStr,1,at("'",cStr)-1) +;
                '&#39'+substr(cStr,at("'",cStr)+1)

      enddo

      do while at('"',cStr) > 0
 
         cStr = substr(cStr,1,at('"',cStr)-1) +;
                '&#34'+substr(cStr,at('"',cStr)+1)

      enddo
      
      ///// rebuild front+text+back

      return cStr
endclass


